/*
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * overlays.c
 *
 * Copyright (C) 2011-2022, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include "cats_global.h"
#include <stdlib.h>

#include <assert.h>
#include "configuration/configuration.h"
#include "logging.h"
#include "memory.h"
#include "overlays.h"
#include "grids/gdal_load.h"
#include "grids/cats_grid.h"
#include "overlay_habitat_type_cc.h"
#include "overlay_exclusion.h"


const char *get_overlay_name(enum overlay_type type)
{
        switch (type) {
                case OL_EXCLUSION:
                        return "exclusion";
                case OL_HABITAT_TYPE_CC:
                        return "habitat_type_cc";

                case OL_NONE:
                case OL_MAX:
                default:
                        break;
        }

        return "unknown overlay type";
}


void cleanup_overlays(struct cats_overlays *overlays)
{
        if (overlays->exclusion) free_cats_grid(&overlays->exclusion);
        if (overlays->habitat_cc) free_cats_grid(&overlays->habitat_cc);
        if (overlays->aux_data[OL_HABITAT_TYPE_CC]) {
                cleanup_habitat_layer_cc_aux(&overlays->aux_data[OL_HABITAT_TYPE_CC]);
        }

        for (int32_t i = 0; i < OL_MAX; i++) {
                free(overlays->filename[i]);
                overlays->filename[i] = NULL;
                free(overlays->filename_aux[i]);
                overlays->filename_aux[i] = NULL;
                free(overlays->filename_loaded[i]);
                overlays->filename_loaded[i] = NULL;
                free(overlays->name[i]);
                overlays->name[i] = NULL;
        }

}


enum overlay_type get_overlay_type_from_name(const char *name)
{
        if (!name) return OL_NONE;

        if (!strcmp(name, "exclusion")) return OL_EXCLUSION;
        if (!strcmp(name, "habitat carrying capacity")) return OL_HABITAT_TYPE_CC;

        log_message(LOG_ERROR, "%s: unknown overlay type '%s'", __func__, name);
        exit(EXIT_FAILURE);
}


void reset_overlay(struct cats_overlays *overlay)
{
        overlay->habitat_cc = NULL;
        overlay->have_overlays = false;
        overlay->exclusion = NULL;
        overlay->cols = 0;
        overlay->rows = 0;
        overlay->excluded_cells = 0;

        for (int32_t i = 0; i < OL_MAX; i++) {
                overlay->enabled[i] = false;
                overlay->is_static[i] = true;
                overlay->reload_interval[i] = 0;
                overlay->filename[i] = NULL;
                overlay->filename_aux[i] = NULL;
                overlay->filename_loaded[i] = NULL;
                overlay->aux_data[i] = NULL;
                overlay->name[i] = NULL;
        }

}


void load_overlay_from_file(struct cats_configuration *conf, enum overlay_type type, char *filename)
{
        assert(conf != NULL);
        if (type <= OL_NONE || type >= OL_MAX) {
                log_message(LOG_ERROR, "overlay type %d out of range", type);
        }
        log_message(LOG_INFO, "gdal: loading grid mask %s", filename);
        struct cats_2d_array_double *raw_values = get_double_values_from_gdal(&conf->geometry, filename, false, false);

        check_raster_dimensions(raw_values->dimension, conf->geometry.dimension);

        if (conf->overlays.filename_loaded[type]) free(conf->overlays.filename_loaded[type]);
        conf->overlays.filename_loaded[type] = strdup(filename);

        switch (type) {
                case OL_EXCLUSION:
                        if (conf->overlays.exclusion) free_cats_grid(&conf->overlays.exclusion);
                        conf->overlays.exclusion = translate_exclusion(raw_values, &conf->overlays.excluded_cells);
                        break;
                case OL_HABITAT_TYPE_CC:
                        if (conf->overlays.habitat_cc) free_cats_grid(&conf->overlays.habitat_cc);
                        conf->overlays.habitat_cc = translate_habitat(raw_values, conf->overlays.aux_data[type]);
                        break;

                case OL_NONE:
                case OL_MAX:
                        break;
        }

        free_cats_grid(&raw_values);
}
